# -*- coding: utf-8 -*-

from django.contrib.auth.decorators import user_passes_test
from django.core.exceptions import PermissionDenied


def has_permissions(user, perm):
    """
    :param user: request.user instance.
    :param perm: tuple of tuples of strings. Example:
        (("account.view_user", "account.autocomplete_user",), ("account.change_user",))
        Inside the inner tuples permissions are evaluated using the AND operator.
        Between tuples there are OR operators.
    """
    if isinstance(perm, str):
        permissions = ((perm,),)
    elif isinstance(perm, tuple) and all(isinstance(x, str) for x in perm):
        permissions = (perm,)
    else:
        permissions = perm
    if not permissions:
        return True
    for permission_batch in permissions:
        if user.has_perms(permission_batch):
            return True
    return False


def permission_required(perm, login_url=None, raise_exception=False):
    """
    Decorator for views that checks whether a user has a particular permission
    enabled, redirecting to the log-in page if necessary.
    If the raise_exception parameter is given the PermissionDenied exception
    is raised.
    """
    def check_perms(user):
        if has_permissions(user, perm):
            return True
        if raise_exception:
            raise PermissionDenied
        return False
    return user_passes_test(check_perms, login_url=login_url)
